unit wave_dlg;

{

   COPYRIGHT 1999 .. 2014 Blue Hell / Jan Punter

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License version 2 as
  published by the Free Software Foundation;

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

  For all listed email addresses :

    _dot. to be substituted by a dot      '.'
    2@t2  to be substituted by an at sign '@'


  Blue Hell is a trade mark owned by

    Jan Punter
    Oogstplein 6
    7545 HP Enschede
    the Netherlands
    http://www.iaf.nl/Users/BlueHell/
    http://bluehell.electro-music.com/
    j_dot.punter2@t2iaf_dot.nl

  All rights attributed to Blue Hell are owned by Jan Punter.
}

interface

uses

  WinApi.Windows, WinApi.Messages, System.SysUtils, System.Variants, System.Classes, Vcl.Graphics, Vcl.Controls,
  Vcl.Forms, Vcl.Dialogs, Vcl.StdCtrls, Vcl.Buttons, WinApi.MMSystem,

  Globals, KnobsUtils, KnobsConversions, portaudio, Vcl.ComCtrls, Vcl.ExtCtrls;

type

  TFormWaveDeviceSelect = class(TForm)
    PanelBottom: TPanel;
    BitBtnOk: TBitBtn;
    BitBtnCancel: TBitBtn;
    PanelMain: TPanel;
    PageControlMain: TPageControl;
    TabSheetClassic: TTabSheet;
    TabSheetPortAudio: TTabSheet;
    PanelPortAudio: TPanel;
    LabelPAVersion: TLabel;
    Label3: TLabel;
    ComboBoxAPI: TComboBox;
    Label4: TLabel;
    Label5: TLabel;
    ComboBoxWaveInPA: TComboBox;
    ComboBoxWaveOutPA: TComboBox;
    Label6: TLabel;
    PanelClassic: TPanel;
    Label1: TLabel;
    Label2: TLabel;
    ComboBoxWaveIn: TComboBox;
    ComboBoxWaveOut: TComboBox;
    LabelInChannels: TLabel;
    LabelOutChannels: TLabel;
    procedure ComboBoxWaveInChange(Sender: TObject);
    procedure ComboBoxWaveOutChange(Sender: TObject);
    procedure BitBtnOkClick(Sender: TObject);
    procedure ComboBoxAPIChange(Sender: TObject);
    procedure ComboBoxWaveInPAChange(Sender: TObject);
    procedure ComboBoxWaveOutPAChange(Sender: TObject);
  private
    FSelectedInput       : string;
    FSelectedOutput      : string;
    FSelectedInputId     : DWORD;
    FSelectedOutputId    : DWORD;
  private
    FSelectedInputPA     : string;
    FSelectedOutputPA    : string;
    FSelectedInputIdPA   : DWORD;
    FSelectedOutputIdPA  : DWORD;
  private
    FInitialized         : Boolean;
  private
    FAllowPortAudio      : Boolean;
    FPADetected          : Boolean;
    FPAInitialized       : Boolean;
    FPAVersion           : string;
    FUsePortAudio        : Boolean;
    FPaApiInfo           : array of TPaHostApiInfo;
    FPaInputDevices      : array of TPaDeviceInfo;
    FPaInputDeviceIDs    : array of TPaDeviceIndex;
    FPaOutputDevices     : array of TPaDeviceInfo;
    FPaOutputDeviceIDs   : array of TPaDeviceIndex;
    FSelectedAPI         : string;
    FSelectedAPIId       : Integer;
  private
    function    ProbePortAudio: Boolean;
    procedure   SetPAVersion( const aValue: string);
    procedure   InitializePortAudio;
    procedure   AddApiInfo( const anApiInfo: PPaHostApiInfo);
    procedure   ClearPAInputDevices;
    procedure   ClearPAOutputDevices;
    procedure   AddPAInputDevice ( anIndex:TPaDeviceIndex; aDeviceInfo: PPaDeviceInfo);
    procedure   AddPAOutputDevice( anIndex:TPaDeviceIndex; aDeviceInfo: PPaDeviceInfo);
    procedure   FillPADevices;
  private
    procedure   SetSelectedInput     ( const aValue: string);
    procedure   SetSelectedOutput    ( const aValue: string);
    procedure   SetSelectedInputId   ( aValue: DWORD);
    procedure   SetSelectedOutputId  ( aValue: DWORD);
  private
    procedure   SetSelectedInputPA   ( const aValue: string);
    procedure   SetSelectedOutputPA  ( const aValue: string);
    procedure   SetSelectedInputIdPA ( aValue: DWORD);
    procedure   SetSelectedOutputIdPA( aValue: DWORD);
  private
    procedure   SetAllowPortAudio    ( aValue: Boolean);
    procedure   SetSelectedAPI       ( const aValue: string);
    procedure   SetSelectedAPIID     ( aValue: Integer);
  private
    function    GetPaInputDevicecount                 : Integer;
    function    GetPaInputDevice   ( anIndex: Integer): TPaDeviceInfo;
    function    GetPaInputDeviceID ( anIndex: Integer): TPaDeviceIndex;
    function    GetPaOutputDevicecount                : Integer;
    function    GetPaOutputDevice  ( anIndex: Integer): TPaDeviceInfo;
    function    GetPaOutputDeviceID( anIndex: Integer): TPaDeviceIndex;
  public
    procedure   Initialize;
    function    Execute: Boolean;
  public
    property    SelectedInput      : string  read FSelectedInput      write SetSelectedInput     ;
    property    SelectedOutput     : string  read FSelectedOutput     write SetSelectedOutput    ;
    property    SelectedInputId    : DWORD   read FSelectedInputId    write SetSelectedInputId   ;
    property    SelectedOutputId   : DWORD   read FSelectedOutputId   write SetSelectedOutputId  ;
  public
    property    SelectedInputPA    : string  read FSelectedInputPA    write SetSelectedInputPA   ;
    property    SelectedOutputPA   : string  read FSelectedOutputPA   write SetSelectedOutputPA  ;
    property    SelectedInputIdPA  : DWORD   read FSelectedInputIdPA  write SetSelectedInputIdPA ;
    property    SelectedOutputIdPA : DWORD   read FSelectedOutputIdPA write SetSelectedOutputIdPA;
  public
    property    AllowPortAudio     : Boolean read FAllowPortAudio     write SetAllowPortAudio;
    property    UsePortAudio       : Boolean read FUsePortAudio;
    property    PADetected         : Boolean read FPADetected;
    property    PAInitialized      : Boolean read FPAInitialized      write FPAInitialized;
    property    PAVersion          : string  read FPAVersion;
    property    SelectedAPI        : string  read FSelectedAPI        write SetSelectedAPI;
    property    SelectedAPIID      : Integer read FSelectedAPIID      write SetSelectedAPIID;
  public
    property    PaInputdeviceCount                  : Integer        read GetPaInputDevicecount;
    property    PaInputDevices   [ anIndex: Integer]: TPaDeviceInfo  read GetPaInputDevice;
    property    PaInputDeviceIDs [ anIndex: Integer]: TPaDeviceIndex read GetPaInputDeviceID;
    property    PaOutputdeviceCount                 : Integer        read GetPaOutputDevicecount;
    property    PaOutputDevices  [ anIndex: Integer]: TPaDeviceInfo  read GetPaOutputDevice;
    property    PaOutputDeviceIDs[ anIndex: Integer]: TPaDeviceIndex read GetPaOutputDeviceID;
  end;

var

  FormWaveDeviceSelect: TFormWaveDeviceSelect;


implementation


{$R *.dfm}


// User area

const

  SApiToUse = '<Select API to use>';


//  private

    function    TFormWaveDeviceSelect.ProbePortAudio: Boolean;
    begin
      Result := False;
      SetPAVersion( 'not detected or not properly initialized');
      if PAInitialized
      then begin
        try
          SetPAVersion( string( Pa_GetVersionText));
          Result := True;
        except
          on E: Exception
          do KilledException( E);
        end;
      end;
      FPADetected := Result;
    end;

    procedure   TFormWaveDeviceSelect.SetPAVersion( const aValue: string);
    begin
      FPAVersion             := aValue;
      LabelPAVersion.Caption := Format( 'Port audio version: %s', [ aValue]);
    end;

    procedure   TFormWaveDeviceSelect.InitializePortAudio;
    var
      i         : Integer;
      aCount    : Integer;
      anApiInfo : PPaHostApiInfo;
    begin
      try
        ClearPAInputDevices;
        ClearPAOutputDevices;

        FSelectedAPIId      := -1;
        FSelectedAPI        := '';
        FSelectedInputIdPA  := DWORD( -1);
        FSelectedOutputIdPA := DWORD( -1);

        aCount := Pa_GetHostApiCount;
        ComboBoxAPI.Clear;
        ComboBoxAPI.Text := SApiToUse;
        for i := 0 to aCount - 1
        do begin
          anApiInfo := Pa_GetHostApiInfo( i);
          if Assigned( anApiInfo)
          then begin
            AddApiInfo( anApiInfo);
            ComboBoxAPI.Items.Add( string( anApiInfo^.name));
          end;
        end;
      except
        on E: Exception
        do begin
          SetPAVersion( 'setup error - could not query API interfaces');
          KilledException( E);
        end;
      end;
    end;

    procedure   TFormWaveDeviceSelect.AddApiInfo( const anApiInfo: PPaHostApiInfo);
    begin
      if Assigned( anApiInfo)
      then begin
        SetLength( FPaApiInfo, Length( FPaApiInfo) + 1);
        FPaApiInfo[ Length( FPaApiInfo) - 1] := anApiInfo^;
      end;
    end;

    procedure   TFormWaveDeviceSelect.ClearPAInputDevices;
    begin
      ComboBoxWaveInPA.Clear; ComboBoxWaveInPA.Text := '';
      SetLength( FPaInputDevices, 0);
    end;

    procedure   TFormWaveDeviceSelect.ClearPAOutputDevices;
    begin
      ComboBoxWaveOutPA.Clear; ComboBoxWaveOutPA.Text := '';
      SetLength( FPaOutputDevices, 0);
    end;

    procedure   TFormWaveDeviceSelect.AddPAInputDevice( anIndex:TPaDeviceIndex; aDeviceInfo: PPaDeviceInfo);
    begin
      SetLength( FPaInputDevices  , PaInputdeviceCount + 1);
      SetLength( FPaInputDeviceIDs, PaInputdeviceCount + 1);
      FPaInputDevices  [ PaInputdeviceCount - 1] := aDeviceInfo^;
      FPaInputDeviceIDs[ PaInputdeviceCount - 1] := anIndex;
      ComboBoxWaveInPA.Items.Add( string( aDeviceInfo.name));
    end;

    procedure   TFormWaveDeviceSelect.AddPAOutputDevice( anIndex:TPaDeviceIndex; aDeviceInfo: PPaDeviceInfo);
    begin
      SetLength( FPaOutputDevices  , PaOutputdeviceCount + 1);
      SetLength( FPaOutputDeviceIDs, PaOutputdeviceCount + 1);
      FPaOutputDevices  [ PaOutputdeviceCount - 1] := aDeviceInfo^;
      FPaOutputDeviceIDs[ PaOutputdeviceCount - 1] := anIndex;
      ComboBoxWaveOutPA.Items.Add( string( aDeviceInfo.name));
    end;

    procedure   TFormWaveDeviceSelect.FillPADevices;
    var
      aCount      : TPaDeviceIndex;
      i           : TPaDeviceIndex;
      aDeviceInfo : PPaDeviceInfo;
    begin
      ClearPAInputDevices;
      ClearPAOutputDevices;
      aCount := Pa_GetDeviceCount;
      for i := 0 to aCount - 1
      do begin
        aDeviceInfo := Pa_GetDeviceInfo( i);
        if aDeviceInfo.hostApi = SelectedAPIID
        then begin
          if aDeviceInfo.maxInputChannels >= 2
          then AddPAInputDevice( i, aDeviceInfo);
          if aDeviceInfo.maxOutputChannels >= 2
          then AddPAOutputDevice( i, aDeviceInfo);
        end;
      end;
    end;

//  private

    procedure   TFormWaveDeviceSelect.SetSelectedInput( const aValue: string);
    var
      i     : Integer;
      Found : Integer;
    begin
      if not FInitialized
      then Initialize;

      if aValue <> FSelectedInput
      then begin
        with ComboBoxWaveIn
        do begin
          Found := -1;
          for i := 0 to Items.Count - 1
          do begin
            if SameText( aValue, Items[ i])
            then begin
              Found := i;
              Break;
            end;
          end;
          if Found >= 0
          then begin
            ItemIndex         := Found;
            Text              := Items[ Found];
            FSelectedInput   := Text;
            FSelectedInputId := Found;
          end;
        end;
      end;
    end;

    procedure   TFormWaveDeviceSelect.SetSelectedOutput( const aValue: string);
    var
      i     : Integer;
      Found : Integer;
    begin
      if not FInitialized
      then Initialize;

      if aValue <> FSelectedOutput
      then begin
        with ComboBoxWaveOut
        do begin
          Found := -1;
          for i := 0 to Items.Count - 1
          do begin
            if SameText( aValue, Items[ i])
            then begin
              Found := i;
              Break;
            end;
          end;
          if Found >= 0
          then begin
            ItemIndex          := Found;
            Text               := Items[ Found];
            FSelectedOutput   := Text;
            FSelectedOutputId := Found;
          end;
        end;
      end;
    end;

    procedure   TFormWaveDeviceSelect.SetSelectedInputId( aValue: DWORD);
    begin
      if not FInitialized
      then Initialize;

      if aValue <> FSelectedInputId
      then begin
        if ( aValue <> DWORD( -1)) and ( Integer( aValue) < ComboBoxWaveIn.Items.Count)
        then SelectedInput := ComboBoxWaveIn.Items[ aValue]
        else begin
          SelectedInput   := '';
          SelectedInputId := DWORD( -1);
        end;
      end;
    end;

    procedure   TFormWaveDeviceSelect.SetSelectedOutputId( aValue: DWORD);
    begin
      if not FInitialized
      then Initialize;

      if aValue <> FSelectedOutputId
      then begin
        if ( aValue <> DWORD( -1)) and ( Integer( aValue) < ComboBoxWaveOut.Items.Count)
        then SelectedOutput := ComboBoxWaveOut.Items[ aValue]
        else begin
          SelectedOutput   := '';
          SelectedOutputId := DWORD( -1);
        end;
      end;
    end;

//  private

    procedure   TFormWaveDeviceSelect.SetSelectedInputPA( const aValue: string);
    var
      i     : Integer;
      Found : Integer;
    begin
      if not FInitialized
      then Initialize;
      if not PADetected
      then Exit;

      if aValue <> FSelectedInputPA
      then begin
        with ComboBoxWaveInPA
        do begin
          Found := -1;
          for i := 0 to Items.Count - 1
          do begin
            if SameText( aValue, Items[ i])
            then begin
              Found := i;
              Break;
            end;
          end;
          if Found >= 0
          then begin
            ItemIndex          := Found;
            Text               := Items[ Found];
            FSelectedInputPA   := Text;
            FSelectedInputIdPA := Found;
          end;
        end;
      end;
      LabelInChannels.Caption := Format( 'channels: %d', [ PaInputDevices[ FSelectedInputIdPA].maxInputChannels]);
    end;

    procedure   TFormWaveDeviceSelect.SetSelectedOutputPA( const aValue: string);
    var
      i     : Integer;
      Found : Integer;
    begin
      if not FInitialized
      then Initialize;
      if not PADetected
      then Exit;

      if aValue <> FSelectedOutputPA
      then begin
        with ComboBoxWaveOutPA
        do begin
          Found := -1;
          for i := 0 to Items.Count - 1
          do begin
            if SameText( aValue, Items[ i])
            then begin
              Found := i;
              Break;
            end;
          end;
          if Found >= 0
          then begin
            ItemIndex           := Found;
            Text                := Items[ Found];
            FSelectedOutputPA   := Text;
            FSelectedOutputIdPA := Found;
          end;
        end;
      end;
      LabelOutChannels.Caption := Format( 'channels: %d', [ PaOutputDevices[ FSelectedOutputIdPA].maxOutputChannels]);
    end;

    procedure   TFormWaveDeviceSelect.SetSelectedInputIdPA( aValue: DWORD);
    begin
      if not FInitialized
      then Initialize;
      if not PADetected
      then Exit;

      if aValue <> FSelectedInputIdPA
      then begin
        if ( aValue <> DWORD( -1)) and ( Integer( aValue) < ComboBoxWaveInPA.Items.Count)
        then SelectedInputPA := ComboBoxWaveInPA.Items[ aValue]
        else begin
          SelectedInputPA    := '';
          FSelectedInputIdPA := DWORD( -1);
        end;
      end;
    end;

    procedure   TFormWaveDeviceSelect.SetSelectedOutputIdPA( aValue: DWORD);
    begin
      if not FInitialized
      then Initialize;
      if not PADetected
      then Exit;

      if aValue <> FSelectedOutputIdPA
      then begin
        if ( aValue <> DWORD( -1)) and ( Integer( aValue) < ComboBoxWaveOutPA.Items.Count)
        then SelectedOutputPA := ComboBoxWaveOutPA.Items[ aValue]
        else begin
          SelectedOutputPA    := '';
          FSelectedOutputIdPA := DWORD( -1);
        end;
      end;
    end;

//  private

    procedure   TFormWaveDeviceSelect.SetAllowPortAudio( aValue: Boolean);
    begin
      if not FInitialized
      then Initialize;
      if not PADetected
      then Exit;

      FAllowPortAudio              := aValue;
      FUsePortAudio                := aValue;
      TabSheetPortAudio.TabVisible := aValue;
    end;

    procedure   TFormWaveDeviceSelect.SetSelectedAPI( const aValue: string);
    var
      i     : Integer;
      Found : Integer;
    begin
      if not FInitialized
      then Initialize;
      if not PADetected
      then Exit;

      if aValue <> FSelectedAPI
      then begin
        with ComboBoxAPI
        do begin
          Found := -1;
          for i := 0 to Items.Count - 1
          do begin
            if SameText( aValue, Items[ i])
            then begin
              Found := i;
              Break;
            end;
          end;
          if Found >= 0
          then begin
            ItemIndex     := Found;
            Text          := Items[ Found];
            FSelectedAPI  := Text;
            SelectedAPIId := Found;
            FillPADevices;
          end
          else ComboBoxAPI.Text := SApiToUse;
        end;
      end;
      LabelInChannels .Caption := 'channels:';
      LabelOutChannels.Caption := 'channels:';
    end;

    procedure   TFormWaveDeviceSelect.SetSelectedAPIID( aValue: Integer);
    begin
      if not FInitialized
      then Initialize;
      if not PADetected
      then Exit;

      if aValue <> FSelectedAPIID
      then begin
        if ( aValue >= 0) and ( aValue < ComboBoxAPI.Items.Count)
        then begin
          FSelectedAPIID := aValue;
          SelectedAPI    := ComboBoxAPI.Items[ aValue];
          FillPADevices;
        end
        else begin
          FSelectedAPIId := -1;
          SelectedAPI    := '';
        end;
      end;
    end;

//  private

    function    TFormWaveDeviceSelect.GetPaInputDevicecount: Integer;
    begin
      Result := Length( FPaInputDevices);
    end;

    function    TFormWaveDeviceSelect.GetPaInputDevice( anIndex: Integer): TPaDeviceInfo;
    begin
      if ( anIndex >= 0) and ( anIndex < PaInputDeviceCount)
      then Result := FPaInputDevices[ anIndex]
      else begin
        Result.hostApi           := -1;
        Result.maxInputChannels  := 0;
        Result.maxOutputChannels := 0;
      end;
    end;

    function    TFormWaveDeviceSelect.GetPaInputDeviceID( anIndex: Integer): TPaDeviceIndex;
    begin
      if ( anIndex >= 0) and ( anIndex < PaInputDeviceCount)
      then Result := FPaInputDeviceIDs[ anIndex]
      else Result := -1;
    end;

    function    TFormWaveDeviceSelect.GetPaOutputDevicecount: Integer;
    begin
      Result := Length( FPaOutputDevices);
    end;

    function    TFormWaveDeviceSelect.GetPaOutputDevice( anIndex: Integer): TPaDeviceInfo;
    begin
      if ( anIndex >= 0) and ( anIndex < PaOutputDeviceCount)
      then Result := FPaOutputDevices[ anIndex]
      else begin
        Result.hostApi           := -1;
        Result.maxInputChannels  := 0;
        Result.maxOutputChannels := 0;
      end;
    end;

    function    TFormWaveDeviceSelect.GetPaOutputDeviceID( anIndex: Integer): TPaDeviceIndex;
    begin
      if ( anIndex >= 0) and ( anIndex < PaOutputDeviceCount)
      then Result := FPaOutputDeviceIDs[ anIndex]
      else Result := -1;
    end;

//  public

    procedure   TFormWaveDeviceSelect.Initialize;
    var
      i         : Integer;
      NumDevs   : Integer;
      InCaps    : WaveInCaps;
      OutCaps   : WaveOutCaps;
      Res       : MMResult;
      Cnt       : Integer;
    begin
      FSelectedInputId  := DWORD( -1);
      FSelectedOutputId := DWORD( -1);

      NumDevs := waveInGetNumDevs;
      Cnt     := 0;
      ComboBoxWaveIn.Items.Clear;
      ComboBoxWaveIn.Text := '';
      for i := 0 to NumDevs
      do begin
        Res := waveInGetDevCaps( i, @ InCaps, SizeOf( WaveInCaps));
        if Res = MMSYSERR_NOERROR
        then begin
          Inc( Cnt);
          with InCaps
          do begin
            ComboBoxWaveIn.Items.Add( Format( '%s', [ szPname], AppLocale));
          end;
        end;
      end;
      if Cnt > 0
      then begin
        ComboBoxWaveIn.Text      := ComboBoxWaveIn.Items[ 0];
        ComboBoxWaveIn.ItemIndex := 1;
      end;

      NumDevs := waveOutGetNumDevs;
      Cnt     := 0;
      ComboBoxWaveOut.Items.Clear;
      ComboBoxWaveOut.Text := '';
      for i := 0 to NumDevs
      do begin
        Res := waveOutGetDevCaps( i, @ OutCaps, SizeOf( WaveOutCaps));
        if Res = MMSYSERR_NOERROR
        then begin
          Inc( Cnt);
          with OutCaps
          do begin
            ComboBoxWaveOut.Items.Add( Format( '%s', [ szPname], AppLocale));
          end;
        end;
      end;
      if Cnt > 0
      then begin
        ComboBoxWaveOut.Text      := ComboBoxWaveOut.Items[ 0];
        ComboBoxWaveOut.ItemIndex := 1;
      end;

      if ProbePortAudio
      then begin
        PageControlMain.ActivePage := TabSheetPortAudio;
        InitializePortAudio;
      end
      else begin
        ComboBoxAPI.Text := '';
        ComboBoxWaveInPA .Clear; ComboBoxWaveInPA .Text := '';
        ComboBoxWaveOutPA.Clear; ComboBoxWaveOutPA.Text := '';
        PageControlMain.ActivePage := TabSheetClassic;
      end;

      FInitialized := True;
    end;


    function    TFormWaveDeviceSelect.Execute: Boolean;
    begin
      if not FInitialized
      then Initialize;

      Result := ShowModal = mrOk;
    end;


// Delphi area


procedure TFormWaveDeviceSelect.BitBtnOkClick(Sender: TObject);
begin
  if PageControlMain.ActivePage = TabSheetClassic
  then FUsePortAudio := False
  else FUsePortAudio := True;
end;

procedure TFormWaveDeviceSelect.ComboBoxAPIChange(Sender: TObject);
begin
  SelectedAPIID := ComboBoxAPI.ItemIndex;
end;

procedure TFormWaveDeviceSelect.ComboBoxWaveInChange(Sender: TObject);
begin
  SelectedInputId := ComboBoxWaveIn.ItemIndex;
end;

procedure TFormWaveDeviceSelect.ComboBoxWaveInPAChange(Sender: TObject);
begin
  SelectedInputIdPA := ComboBoxWaveInPA.ItemIndex;
end;

procedure TFormWaveDeviceSelect.ComboBoxWaveOutChange(Sender: TObject);
begin
  SelectedOutputId := ComboBoxWaveOut.ItemIndex;
end;


procedure TFormWaveDeviceSelect.ComboBoxWaveOutPAChange(Sender: TObject);
begin
  SelectedOutputIdPA := ComboBoxWaveOutPA.ItemIndex;
end;

end.

